import { Component, OnInit, OnDestroy } from '@angular/core';
import {  Router, ActivatedRoute } from '@angular/router';
import * as _ from 'lodash';
import { CommonService, StoreService } from 'services';
import { OrderApi, MyOrderListItem, MyOrderData } from 'client';
import { MxWinAlert } from 'interfaces';
import { CAN_WX_PAY } from 'consts';

@Component({
  selector: 'order-list',
  templateUrl: './list.component.html',
  styles: [require('./list.component.scss')],
  providers: [CommonService, StoreService]
})
export class OrderListComponent implements OnInit {
  public menu = {
    show: false
  };

  public lists: MyOrderListItem[] = [];
  public listLoading: boolean = false;
  public currentTab: number = 0;
  public viewOrder: MyOrderData;
  public invoiceOrderId: number;
  public showAlertWin: boolean = false;
  public winConfirm = {
    msg: '',
    type: ''
  };

  private token: string = '';
  private currentDelOrder: MyOrderListItem;

  private applyInvoiceAlertInfo: MxWinAlert = {
    tip: '申请成功',
    subTip: '稍后我们的客服会联系您获取材料',
    icon: 'cssico-right-okey-64',
    btnText: '确定'
  };

  constructor(
    public ss: StoreService,
    private router: Router,
    private route: ActivatedRoute,
    private cs: CommonService,
    private oApi: OrderApi
  ) {

  }

  public ngOnInit() {
    this.token = localStorage.getItem('token') || '';
    this.getOrderList();
  }

  /**
   * 切换标签项
   * 
   * @param {any} tag 
   * 
   * @memberOf OrderListComponent
   */
  public onToggleTab(tag) {
    this.currentTab = tag;
  }

  /**
   * 查看订单详情
   * 
   * @param {any} item 
   * 
   * @memberOf OrderListComponent
   */
  public onViewOrderDetail(item) {
    this.oApi.orderCarMobileOrderIdGet(item.id).subscribe((data) => {
      if (data.meta.code === 200) {
        this.viewOrder = data.data;
      } else {
        this.cs.responseErrorHandler(data, true, this.router);
      }
    }, (err) => console.error(err));

  }

  /**
   * 到订单支付页
   * 
   * @param {any} item 
   * @param {any} evt 
   * 
   * @memberOf OrderListComponent
   */
  public onPayOrder(item, evt) {
    evt.stopPropagation();
    if (CAN_WX_PAY) {
      this.cs.requestOpenIdURL(item.id, 'orderList');
    } else {
      this.router.navigate([`/storeorder`], {queryParams: {
          openId: '1233',
          orderId: String(item.id),
          source: 'orderList'
      }});
    }
  }

  /**
   * 显示申请发票窗口
   * 
   * @param {any} item 
   * @param {any} evt 
   * 
   * @memberOf OrderListComponent
   */
  public onApplyInvoice(item, evt) {
    evt.stopPropagation();
    this.invoiceOrderId = item.id;
  }

  /**
   * 通过tab筛选订单
   * 
   * @returns 
   * 
   * @memberOf OrderListComponent
   */
  public getOrderByTab() {
    if (this.currentTab === 0) {
      return this.lists;
    }
    return this.lists.filter((item) => item.status === this.currentTab);
  }

  /**
   * 保存发票信息成功
   * 
   * 
   * @memberOf OrderListComponent
   */
  public onSaveInvoiceSuc() {
    let idx = _.findIndex(this.lists, ['id', this.invoiceOrderId]);
    this.lists[idx].invoiceStatus = 2;
    this.invoiceOrderId = undefined;
    this.showAlertWin = true;
  }

  /**
   * 取消申请发票
   * 
   * 
   * @memberOf OrderListComponent
   */
  public onCancelInvoice() {
    this.invoiceOrderId = undefined;
  }

  /**
   * 关闭提示层
   * 
   * 
   * @memberOf OrderListComponent
   */
  public onCloseAlertWin() {
    this.showAlertWin = false;
  }

  /**
   * 关闭订单详情窗口
   * 
   * @param {any} evt 
   * 
   * @memberOf OrderListComponent
   */
  public onCloseDetailWin(evt) {
    this.viewOrder = undefined;
  }

  /**
   * 订单 - 点击删除按钮
   * 
   * @private
   * 
   * @memberOf StoreEditComponent
   */
  public onDelOrder(item, evt) {
    evt.stopPropagation();
    this.winConfirm = {
      msg: '订单删除后，将无法恢复。是否删除？',
      type: 'delete'
    };
    this.currentDelOrder = item;
  }

  /**
   * 警告 - 确认操作
   * 
   * @private
   * @param {any} evt 
   * @returns 
   * 
   * @memberOf StoreEditComponent
   */
  public onConfirmOkeyHandle(evt) {
    if (this.winConfirm.type === 'delete') {
      this.delOrder(this.currentDelOrder);
      this.winConfirm = { msg: '', type: '' };
      return;
    }
  }

  /**
   * 警告 - 取消操作
   * 
   * @private
   * 
   * @memberOf StoreEditComponent
   */
  public onConfirmCancelHandle(evt) {
    this.winConfirm = { msg: '', type: '' };
  }

  /**
   * 显示主菜单
   * 
   * 
   * @memberOf ReportComponent
   */
  public onShowMenu() {
    this.menu.show = true;
  }

  /**
   * 获取订单列表
   * 
   * @private
   * 
   * @memberOf OrderListComponent
   */
  private getOrderList() {
    this.listLoading = true;
    this.oApi.orderCarMobileListGet(this.token).subscribe((data) => {
      if (data.meta.code === 200) {
        this.lists = data.data;
      } else {
        this.cs.responseErrorHandler(data, true, this.router);
      }
    }, (err) => console.error(err), () => {
      this.listLoading = false;
    });

  }

  /**
   * 删除订单
   * 
   * @param {any} item 
   * @param {any} evt 
   * 
   * @memberOf OrderListComponent
   */
  private delOrder(item) {
    this.oApi.orderCarMobileIdDeleteDelete(item.id).subscribe((data) => {
      if (data.meta.code === 200) {
        let idx = _.findIndex(this.lists, ['id', item.id]);
        this.lists.splice(idx, 1);
        this.currentDelOrder = undefined;
      } else {
        this.cs.responseErrorHandler(data, true, this.router);
      }
    }, (err) => console.error(err));

  }

}
