import {
  Component,
  OnInit,
  OnDestroy
} from '@angular/core';
import {  Router, ActivatedRoute } from '@angular/router';
import * as _ from 'lodash';
import { CommonService, StoreService, MxUtilService } from 'services';
import { MxStore } from 'interfaces';
import { ShopApi, BatchSaveShopItem } from 'client';
import { CAN_WX_PAY } from 'consts';

@Component({
  selector: 'store-add',
  templateUrl: './add.component.html',
  styles: [require('./add.component.scss')],
  providers: [CommonService, StoreService, ShopApi]
})
export class StoreAddComponent {
  public menu = {
    show: false
  };
  public buyServiceTypes: any[] = [];

  private store: MxStore;
  private errorMsg: string = '';
  private submitting: boolean = false;

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private cs: CommonService,
    private ss: StoreService,
    private mus: MxUtilService,
    private sApi: ShopApi
  ) {
    this.store = this.ss.getDefaultStoreValue();
    this.buyServiceTypes = this.ss.getStoreBuyServiceList();
  }

  /**
   * 显示主菜单
   * 
   * 
   * @memberOf ReportComponent
   */
  public onShowMenu() {
    this.menu.show = true;
  }

  private onErrorMsg(evt) {
    this.errorMsg = evt;
  }

  private onSaveStore() {
    if (this.submitting) { return; }
    if (!this.checkStoreFormInfo()) { return; }
    this.submitting = true;
    this.sApi.shopCarMobileBatchSavePost({shopList: [this.formatStoreData(this.store)], viewKey: this.ss.getStoreBuyServiceKey(this.store.money)}).subscribe((data) => {
      if (data.meta.code === 200) {
        if (CAN_WX_PAY) {
          this.cs.requestOpenIdURL(Number(data.data), 'storeAdd');
        } else {
          this.router.navigate([`/storeorder`], {queryParams: {
            openId: '1233',
            orderId: String(data.data),
            source: 'storeAdd'
          }});
        }
      } else {
        if (data.error && data.error.message) {
          this.cs.responseErrorHandler(data, false, this.router);
        }
      }
    }, (err) => console.error(err), () => {
      this.submitting = false;
    });
  }

  /**
   * 验证门店字段
   * 
   * @private
   * @returns {boolean} 
   * 
   * @memberOf StoreAddComponent
   */
  private checkStoreFormInfo(): boolean {
    let { name, provinceId, cityId, address, serviceType, manager, tel } = this.store;
    if (this.mus.isEmptyString(name)) {
      this.errorMsg = '请输入门店名称';
      return false;
    }
    if (this.mus.isEmptyString(String(provinceId)) || this.mus.isEmptyString(String(cityId)) || this.mus.isEmptyString(address)) {
      this.errorMsg = '请完善门店地址';
      return false;
    }
    if (this.mus.isEmptyString(manager)) {
      this.errorMsg = '请输入负责人';
      return false;
    }
    if (this.mus.isEmptyString(tel)) {
      this.errorMsg = '请输入联系方式';
      return false;
    }
    if (!(this.mus.isPhoneNumber(tel) || this.mus.isHandFreePhoneNumber(tel))) {
        this.errorMsg = '请输入正确的联系方式';
        return false;
    }
    if (this.mus.isEmptyString(serviceType)) {
      this.errorMsg = '请选择服务类型';
      return false;
    }
    this.errorMsg = '';
    return true;
  }

  /**
   * 格式化提交的门店数据
   * 
   * @private
   * @param {MxStore} store 
   * @returns {BatchSaveShopItem} 
   * 
   * @memberOf StoreAddComponent
   */
  private formatStoreData(store: MxStore): BatchSaveShopItem {
    return {
      name: store.name.trim(),
      provinceId: Number(store.provinceId),
      cityId: Number(store.cityId),
      address: store.address.trim(),
      serviceType: store.serviceType.trim(),
      manager: store.manager.trim(),
      tel: store.tel.trim(),
      money: store.money
    };

  }

  /**
   * 返回门店列表页
   * 
   * @private
   * 
   * @memberOf StoreAddComponent
   */
  private gotoStoreListPage() {
    this.cs.gotoListPage(this.router, 'store');
  }

}
