import {
  Component,
  OnInit,
  OnDestroy
} from '@angular/core';
import {  Router, ActivatedRoute } from '@angular/router';
import * as _ from 'lodash';
import { CommonService, StoreService, MxUtilService } from 'services';
import { MxStore } from 'interfaces';
import { ShopApi, UserApi, UpdateShopInfoObj } from 'client';

@Component({
  selector: 'store-edit',
  templateUrl: './edit.component.html',
  styles: [require('./edit.component.scss')],
  providers: [CommonService, StoreService]
})
export class StoreEditComponent implements OnInit, OnDestroy {
  public menu = {
    show: false
  };
  public isSub: number = 0;
  private sub: any;
  private storeId: number;
  private store: MxStore;
  private errorMsg: string = '';
  private submitting: boolean = false;
  private winConfirm = {
    msg: '',
    type: ''
  };
  private token = '';

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private cs: CommonService,
    private ss: StoreService,
    private mus: MxUtilService,
    private sApi: ShopApi,
    private uApi: UserApi
  ) {
    this.store = this.ss.getDefaultStoreValue();
  }

  public ngOnInit() {
    this.token = localStorage.getItem('token') || '';
    this.isSub = Number(localStorage.getItem('isSub'));
    this.sub = this.route.params.subscribe((data) => {
      if (data['storeId']) {
        this.storeId = +data['storeId'];
        this.getShopInfoById(this.storeId);
      }
    });
  }

  public ngOnDestroy() {
    this.sub.unsubscribe();
  }

  /**
   * 显示主菜单
   * 
   * 
   * @memberOf ReportComponent
   */
  public onShowMenu() {
    this.menu.show = true;
  }

  private onErrorMsg(evt) {
    this.errorMsg = evt;
  }

  private onSaveStore() {
    if (this.submitting) { return; }
    if (!this.checkStoreFormInfo()) { return; }
    this.submitting = true;
    this.sApi.shopCarMobileUpdatePost(this.formatStoreData(this.store)).subscribe((data) => {
      if (data.meta.code === 200) {
        this.cs.gotoListPage(this.router, 'store');
      } else {
        if (data.error && data.error.message) {
          this.cs.responseErrorHandler(data, true, this.router);
        }
      }
    }, (err) => console.error(err), () => {
      this.submitting = false;
    });
  }

  /**
   * 验证门店字段
   * 
   * @private
   * @returns {boolean} 
   * 
   * @memberOf StoreAddComponent
   */
  private checkStoreFormInfo(): boolean {
    let { name, provinceId, cityId, address, serviceType, manager, tel } = this.store;
    if (this.mus.isEmptyString(name)) {
      this.errorMsg = '请输入门店名称';
      return false;
    }
    if (this.mus.isEmptyString(String(provinceId)) || this.mus.isEmptyString(String(cityId)) || this.mus.isEmptyString(address)) {
      this.errorMsg = '请完善门店地址';
      return false;
    }
    if (this.mus.isEmptyString(manager)) {
      this.errorMsg = '请输入负责人';
      return false;
    }
    if (this.mus.isEmptyString(tel)) {
      this.errorMsg = '请输入联系方式';
      return false;
    }
    if (!(this.mus.isPhoneNumber(tel) || this.mus.isHandFreePhoneNumber(tel))) {
        this.errorMsg = '请输入正确的联系方式';
        return false;
    }
    if (this.mus.isEmptyString(serviceType)) {
      this.errorMsg = '请选择服务类型';
      return false;
    }
    this.errorMsg = '';
    return true;
  }

  /**
   * 格式化提交的门店数据
   * 
   * @private
   * @param {MxStore} store 
   * @returns {BatchSaveShopItem} 
   * 
   * @memberOf StoreAddComponent
   */
  private formatStoreData(store: MxStore): UpdateShopInfoObj {
    return {
      id: Number(store.id),
      name: store.name.trim(),
      provinceId: Number(store.provinceId),
      cityId: Number(store.cityId),
      address: store.address.trim(),
      serviceType: store.serviceType.trim(),
      manager: store.manager.trim(),
      tel: store.tel.trim()
    };

  }

  /**
   * 通过门店 Id 查询门店信息
   * 
   * @private
   * @param {number} id 
   * 
   * @memberOf StoreEditComponent
   */
  private getShopInfoById(id: number) {
    this.sApi.shopCarMobileShopIdGet(id).subscribe((data) => {
      if (data.meta.code === 200) {
        this.store = Object.assign({}, this.store, data.data);
      } else {
        this.cs.responseErrorHandler(data, false, this.router);
      }
    }, (err) => console.error(err));
  }

  /**
   * 门店 - 点击删除门店按钮
   * 
   * @private
   * 
   * @memberOf StoreEditComponent
   */
  private onDeleteStore() {
    this.winConfirm = {
      msg: '门店删除后，将无法恢复。是否删除？',
      type: 'delete'
    };
  }

  /**
   * 门店 - 删除门店
   * 
   * @private
   * 
   * @memberOf StoreEditComponent
   */
  private deleteStore() {
    this.sApi.shopCarMobileDeleteDelete(Number(this.store.id)).subscribe((data) => {
      if (data.meta.code === 200) {
        this.uApi.userUserInfoTokenGet(this.token).subscribe((dd) => {
          if (dd.meta.code === 200) {
            localStorage.setItem('shopId', String(dd.data.shopId));
            localStorage.setItem('shopPay', String(dd.data.pay));
            this.cs.gotoListPage(this.router, 'store');
          }
        }, (err) => console.error(err));
      } else {
        this.cs.responseErrorHandler(data, true, this.router);
      }
    }, (err) => console.error(err));
  }

  /**
   * 警告 - 确认操作
   * 
   * @private
   * @param {any} evt 
   * @returns 
   * 
   * @memberOf StoreEditComponent
   */
  private onConfirmOkeyHandle(evt) {
    if (this.winConfirm.type === 'delete') {
      this.deleteStore();
      this.winConfirm = { msg: '', type: '' };
      return;
    }
  }

  /**
   * 警告 - 取消操作
   * 
   * @private
   * 
   * @memberOf StoreEditComponent
   */
  private onConfirmCancelHandle(evt) {
    this.winConfirm = { msg: '', type: '' };
  }

  /**
   * 返回门店列表页
   * 
   * @private
   * 
   * @memberOf StoreAddComponent
   */
  private gotoStoreListPage() {
    this.cs.gotoListPage(this.router, 'store');
  }

}
