import {
  Component,
  OnInit,
  OnDestroy
} from '@angular/core';
import {  Router, ActivatedRoute } from '@angular/router';
import * as _ from 'lodash';
import { CommonService, StoreService, MxUtilService, SubAccountService } from 'services';
import { MxSubAccount } from 'interfaces';
import { ShopApi, UserApi, UpdateShopInfoObj } from 'client';

@Component({
  selector: 'sub-account-edit',
  templateUrl: './edit.component.html',
  styles: [require('./edit.component.scss')],
  providers: [CommonService, StoreService, ShopApi, SubAccountService]
})
export class SubAccountEditComponent implements OnInit, OnDestroy {
  public menu = {
    show: false
  };

  // 处理 iphone 手机浏览器 fixed 超不出顶部bug
  public showShopSelect: boolean = false;

  private sub: any;
  private accountId: number;
  private account: MxSubAccount;
  private errorMsg: string = '';
  private submitting: boolean = false;
  private winConfirm = {
    msg: '',
    type: ''
  };

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private cs: CommonService,
    private ss: StoreService,
    private mus: MxUtilService,
    private sApi: ShopApi,
    private sas: SubAccountService,
    private uApi: UserApi
  ) {
    this.account = this.sas.getDefaultSubAccountValue();
  }

  public ngOnInit() {
    this.sub = this.route.params.subscribe((data) => {
      if (data['accountId']) {
        this.accountId = +data['accountId'];
        this.getAccountInfoById(this.accountId);
      }
    });
  }

  public ngOnDestroy() {
    this.sub.unsubscribe();
  }

  /**
   * 显示主菜单
   * 
   * 
   * @memberOf ReportComponent
   */
  public onShowMenu() {
    this.menu.show = true;
  }

  public onVisibleSelectShop(evt) {
    this.showShopSelect = evt;
  }

  private onErrorMsg(evt) {
    this.errorMsg = evt;
  }

  private onSave() {
    if (this.submitting) { return; }
    if (!this.validateFormInfo()) { return; }
    this.submitting = true;
    this.uApi.userCarMobileAccountUpdatePost(Number(this.account.id), '[' + this.account.shopId.trim() + ']', this.account.name.trim()).subscribe((data) => {
      if (data.meta.code === 200) {
        this.gotoSubAccountListPage();
      } else {
        if (data.error && data.error.message) {
          this.cs.responseErrorHandler(data, false, this.router);
        }
      }
    }, (err) => console.error(err), () => {
      this.submitting = false;
    });
  }

  /**
   * 验证表单字段
   * 
   * @private
   * @returns {boolean} 
   * 
   * @memberOf StoreAddComponent
   */
  private validateFormInfo(): boolean {
    let { name, mobile, shopId } = this.account;

    if (this.mus.isEmptyString(mobile)) {
      this.errorMsg = '登录账号不能为空';
      return false;
    }
    if (!this.mus.isPhoneNumber(mobile)) {
      this.errorMsg = '请输入正确的手机号码';
      return false;
    }
    if (this.mus.isEmptyString(shopId)) {
      this.errorMsg = '请选择门店';
      return false;
    }
    this.errorMsg = '';
    return true;
  }

  /**
   * 通过账户 Id 查询账户信息
   * 
   * @private
   * @param {number} id 
   * 
   * @memberOf StoreEditComponent
   */
  private getAccountInfoById(id: number) {
    this.uApi.userCarMobileAccountIdGet(id).subscribe((data) => {
      if (data.meta.code === 200) {
        data.data.shopId = data.data.shopId.substr(1);
        data.data.shopId = data.data.shopId.substr(0, data.data.shopId.length - 1);

        this.account = Object.assign({}, this.account, data.data);
      } else {
        this.cs.responseErrorHandler(data, false, this.router);
      }
    }, (err) => console.error(err));
  }

  /**
   * 账户 - 点击删除账户按钮
   * 
   * @private
   * 
   * @memberOf StoreEditComponent
   */
  private onDeleteAccount() {
    this.winConfirm = {
      msg: '子账户删除后，将无法恢复。是否删除？',
      type: 'delete'
    };
  }

  /**
   * 账户 - 删除账户
   * 
   * @private
   * 
   * @memberOf StoreEditComponent
   */
  private deleteAccount() {
    this.uApi.userCarMobileAccountIdDeleteDelete(Number(this.account.id)).subscribe((data) => {
      if (data.meta.code === 200) {
        this.gotoSubAccountListPage();
      } else {
        this.cs.responseErrorHandler(data, false, this.router);
      }
    }, (err) => console.error(err));
  }

  /**
   * 警告 - 确认操作
   * 
   * @private
   * @param {any} evt 
   * @returns 
   * 
   * @memberOf StoreEditComponent
   */
  private onConfirmOkeyHandle(evt) {
    if (this.winConfirm.type === 'delete') {
      this.deleteAccount();
      this.winConfirm = { msg: '', type: '' };
      return;
    }
  }

  /**
   * 警告 - 取消操作
   * 
   * @private
   * 
   * @memberOf StoreEditComponent
   */
  private onConfirmCancelHandle(evt) {
    this.winConfirm = { msg: '', type: '' };
  }

  /**
   * 返回门店列表页
   * 
   * @private
   * 
   * @memberOf StoreAddComponent
   */
  private gotoSubAccountListPage() {
    this.cs.gotoListPage(this.router, 'subaccount');
  }

}
