import {
  Component,
  OnInit,
  NgZone
} from '@angular/core';
import {  Router, ActivatedRoute } from '@angular/router';
import { CommonService } from 'services';
import { UserApi, CommonApi, User } from 'client';

// https://github.com/cotag/ts-md5
import { Md5 } from 'ts-md5/dist/md5';
// import { Cookie } from 'ng2-cookies';

@Component({
  selector: 'user-regist',
  templateUrl: './regist.component.html',
  styleUrls: ['./regist.component.scss'],
  providers: [ CommonApi ]
})
export class UserRegistComponent implements OnInit {
  public zone: any;
  public step: number = 1;
  public img: string = '';
  public uuid: string = '';
  public regForm: any = {
    mobile: '',
    code: '',
    captcha: '',
    password: '',
    registerChannal: 1,
    type: 2,
    protocol: true
  };
  public errorMsg: string = '';
  // 发短信
  public freezing: boolean = false;
  public freezeTimeout: any;
  public freezeTime: number = 60;
  public freezeBtnTxt: string = '获取验证码';
  // 协议
  public openProtocol: boolean = false;
  private submitting: boolean = false;
  private salt: string = 'thzs0708';

  public constructor(
    private router: Router,
    private uApi: UserApi,
    private cApi: CommonApi,
    private cs: CommonService
  ) {
    this.zone = new NgZone({ enableLongStackTrace: false });
  }

  public ngOnInit() {
    this.onGetCodeImg();
  }

  /**
   * 生成图片验证码
   * 
   * 
   * @memberOf UserRegistComponent
   */
  public onGetCodeImg() {
    this.cApi.commonCaptchaBase64Post().subscribe((data) => {
      const resbody = data.json();
      if (resbody.meta.code === 200) {
        this.img = 'data:image/jpeg;base64,' + (resbody.data || '');
        this.uuid = data.headers.get('uuid');
        this.uApi.defaultHeaders.set('uuid', data.headers.get('uuid'));
      }
    });
  }

  /**
   * 短信验证码 - 获取短信验证码
   * 
   * 
   * @memberOf UserRegistComponent
   */
  public getSmsCode() {
    // 是否处于冻结状态
    if (this.freezing) {
      return false;
    }
    // 检查手机号和图形验证码
    if (!this.checkMobileAndApatcha()) {
      return false;
    }
    // 创建60s计时器
    this.createFreezeTimer();
    let rnd = Math.random().toString(16).slice(2, 6);
    let sign = Md5.hashStr(this.regForm.mobile + rnd + this.salt).toString();
    this.uApi.userRegisterSmsPost(this.regForm.mobile, rnd, this.regForm.captcha, sign).subscribe((data) => {
      if (data.meta.code !== 200) {
        if (this.freezeTimeout) {
          clearInterval(this.freezeTimeout);
        }
        if (data.error && data.error.message) {
          this.errorMsg = data.error.message;
          this.freezeBtnTxt = '重新发送';
          this.freezing = false;
        }
      }
    }, (err) => console.error(err));
  }

  /**
   * 注册新用户
   * 
   * @returns
   * 
   * @memberOf UserRegistComponent
   */
  public onRegist() {
    if (this.submitting) {
      return;
    }
    if (!this.checkAllFields()) {
      return;
    }
    this.submitting = true;
    let {mobile, password, code, captcha, registerChannal, type} = this.regForm;
    this.uApi.userRegisterPost(mobile, Md5.hashStr(password, false).toString(), code, this.uuid, captcha, registerChannal, type).subscribe((data) => {
      if (data.meta.code === 200) {
        this.cs.removeUserInfoFromLocalStorage();
        // Cookie.set('token', data.data.token, 14);
        window.localStorage.setItem('token', data.data.token);
        window.localStorage.setItem('isSub', '0');
        window.localStorage.setItem('subStatus', '1');
        this.cs.setHeardFieldForApi('token', data.data.token);
        this.router.navigate(['/account/store/init']);
      } else {
        if (data.error && data.error.message) {
          this.errorMsg = data.error.message === '手机号已被注册' || data.error.message === '该手机号码已注册，请登录' ? `该手机号码已注册，请<a class="c-green" routerLink="/login">登录</a>` : data.error.message;
        }
      }
    }, (err) => console.error(err), () => {
      this.submitting = false;
    });
  }

  /**
   * 表单字段获取焦点
   * 
   * 
   * @memberOf UserRegistComponent
   */
  public onFieldFocus() {
    this.errorMsg = '';
  }

  /**
   * 转入注册表单界面
   * 
   * 
   * @memberOf UserRegistComponent
   */
  public onGotoRegForm() {
    this.step = 2;
  }

  /**
   * 验证手机号和图形验证码
   * 
   * @private
   * @returns {boolean}
   * 
   * @memberOf UserRegistComponent
   */
  private checkMobileAndApatcha(): boolean {
    let mobile = this.regForm.mobile = this.regForm.mobile.trim();
    let captcha = this.regForm.captcha = this.regForm.captcha.trim();
    let mobileReg = /^(13[0-9]|15[012356789]|17[0135678]|18[0-9]|14[57])[0-9]{8}$/;
    if (mobile === '') {
      this.errorMsg = '手机号不能为空';
      return false;
    }
    if (!mobileReg.test(mobile)) {
      this.errorMsg = '请输入正确的手机号';
      return false;
    }
    if (captcha === '') {
      this.errorMsg = '验证码不能为空';
      return false;
    }
    this.errorMsg = '';
    return true;
  }

  /**
   * 检查注册表单的所有字段
   * 
   * @private
   * @returns
   * 
   * @memberOf UserRegistComponent
   */
  private checkAllFields() {
    if (!this.regForm.protocol) {
      return false;
    }
    if (!this.checkMobileAndApatcha()) {
      return false;
    }
    let code = this.regForm.code = this.regForm.code.trim();
    let password = this.regForm.password = this.regForm.password.trim();
    if (code === '') {
      this.errorMsg = '短信验证码不能为空';
      return false;
    }
    if (password === '') {
      this.errorMsg = '密码不能为空';
      return false;
    }
    if (password.length < 6 || password.length > 16) {
      this.errorMsg = '请输入由6~16位的英文字母,数字和符号组成的密码';
      return false;
    }
    if (/^(?![a-zA-z]+$)(?!\d+$)(?![!@#$%^&*]+$)(?![a-zA-z\d]+$)(?![a-zA-z!@#$%^&*]+$)(?![\d!@#$%^&*]+$)[a-zA-Z\d!@#$%^&*]+$/.test(password)) {
      this.errorMsg = '请输入由6~16位的英文字母,数字和符号组成的密码';
      return false;
    }
    this.errorMsg = '';
    return true;
  }

  /**
   * 创建倒数计时器
   * 
   * @private
   * 
   * @memberOf UserRegistComponent
   */
  private createFreezeTimer() {
    this.freezing = true;
    this.freezeTime = 60;
    this.freezeBtnTxt = `${this.freezeTime}s`;
    this.freezeTimeout = window.setInterval(() => {
      this.zone.run(() => {
        if (this.freezeTime > 1) {
          this.freezeTime--;
          this.freezeBtnTxt = `${this.freezeTime}s`;
        } else {
          this.freezeBtnTxt = `重新发送`;
          this.freezing = false;
          clearInterval(this.freezeTimeout);
        }
      });
    }, 1000);
  }

}
