import { Injectable, Inject } from '@angular/core';
import { PayApi, UserApi, ShopApi, OrderApi, DefaultApi } from 'client';

@Injectable()
export class CommonService {
    constructor(
        @Inject(PayApi) protected pApi: PayApi,
        @Inject(UserApi) protected uApi: UserApi,
        @Inject(ShopApi) protected sApi: ShopApi,
        @Inject(OrderApi) protected oApi: OrderApi,
        @Inject(DefaultApi) protected dApi: DefaultApi
    ) {
        console.log('CommonService constructor');
        this.initTokenForApiService();
        this.initChannelForApiService();
    }
    public checkObjectFieldEmpty(obj, key, msg) {
        obj[key] = obj[key].trim();
        return obj[key] === '' ? false : true;
    }

    /**
     * API：为api接口类头部添加 字段
     *
     * @param {string} key
     * @param {string} value
     *
     * @memberOf CommonService
     */
    public setHeardFieldForApi(key: string, value: string): void {
        this.pApi.defaultHeaders.set(key, value);
        this.uApi.defaultHeaders.set(key, value);
        this.sApi.defaultHeaders.set(key, value);
        this.oApi.defaultHeaders.set(key, value);
        this.dApi.defaultHeaders.set(key, value);
    }

    /**
     * 设置用户信息到 localStorage
     *
     * @param {any} data
     *
     * @memberOf CommonService
     */
    public setUserInfoToLocalStorage(data) {
        localStorage.setItem('token', data.token);
        localStorage.setItem('shopId', String(data.shopId));
        localStorage.setItem('shopPay', String(data.pay));
        localStorage.setItem('isSub', String(data.isSub));
        localStorage.setItem('subStatus', String(data.subStatus));
    }

    /**
     * 删除用户信息从 localStorage
     *
     *
     * @memberOf CommonService
     */
    public removeUserInfoFromLocalStorage() {
        localStorage.removeItem('token');
        localStorage.removeItem('shopId');
        localStorage.removeItem('shopPay');
        localStorage.removeItem('isSub');
        localStorage.removeItem('subStatus');
    }

    /**
     * 初始化 api headers token
     *
     *
     * @memberOf CommonService
     */
    public initTokenForApiService() {
        let token = localStorage.getItem('token');
        this.setHeardFieldForApi('token', token || '');
    }
    /**
     * 初始化 api headers channel
     *
     *
     * @memberOf CommonService
     */
    public initChannelForApiService() {
        this.setHeardFieldForApi('channel', 'carh5');
    }

    /**
     * 跳转到登录页
     *
     * @param {any} router
     *
     * @memberOf CommonService
     */
    public gotoLoginPage(router) {
        router.navigate([`/user/login`]);
    }

    /**
     * 跳转到门店编辑页
     *
     * @param {any} router
     * @param {any} storeId
     *
     * @memberOf CommonService
     */
    public gotoStoreEditPage(router, storeId) {
        router.navigate([`/dashboard/store/edit/${storeId}`]);
    }

    /**
     * 返回开发中页面
     * 
     * @param {any} router 
     * 
     * @memberOf CommonService
     */
    public gotoDevPage(router) {
        router.navigate([`/temp/page/business`]);
    }

    /**
     * 跳转 - 模块项目编辑页
     *
     * @param {any} router
     * @param {any} group
     * @param {any} groupId
     *
     * @memberOf CommonService
     */
    public gotoEditPage(router, group, groupId) {
        router.navigate([`/dashboard/${group}/edit/${groupId}`]);
    }

    /**
     * 跳转到模块项添加页
     *
     * @param {any} router
     *
     * @memberOf CommonService
     */
    public gotoCreatePage(router, group: string) {
        router.navigate([`/dashboard/${group}/add`]);
    }

    /**
     * 跳转 - 跳转到模块列表页
     *
     * @param {any} router
     *
     * @memberOf CommonService
     */
    public gotoListPage(router, group: string) {
        router.navigate([`/dashboard/${group}/list`]);
    }

    /**
     * 跳转 - 子账户激活
     *
     * @param {any} router
     *
     * @memberOf CommonService
     */
    public gotoActiveSubAccountPage(router) {
        router.navigate([`/activepwd`]);
    }

    /**
     * 跳转 - 报告中心
     *
     * @param {any} router
     *
     * @memberOf CommonService
     */
    public gotoReportCenterPage(router) {
        router.navigate([`/dashboard/report/store`]);
    }

    /**
     * 跳转 - 初始化门店
     *
     * @param {any} router
     *
     * @memberOf CommonService
     */
    public gotoInitStorePage(router) {
        router.navigate([`/account/store/init`]);
    }

    public gotoTurnplatePage(router) {
        router.navigate([`/award/turnplate`]);
    }

    /**
     * 接口调用失败的处理函数
     *
     * @param {any} data
     * @param {boolean} [tip=true]
     * @param {any} [router=null]
     * @returns
     *
     * @memberOf CommonService
     */
    public responseErrorHandler(data, tip = true, router = null) {
        if (data.error && data.error.code === 401 && router) {
            this.gotoLoginPage(router);
            return;
        }
        if (data.error && data.error.message && tip) {
            alert(data.error.message);
            return;
        }
        return data.error && data.error.message ? data.error.message : '';
    }

    /**
     * 请求OpenId， 并跳转到支付页
     * 
     * @param {any} orderid 
     * @param {any} source 
     * 
     * @memberOf CommonService
     */
    public requestOpenIdURL(orderid, source) {
        this.pApi.weChatOpenIdURLGet().subscribe((data) => {
            let host = 'http://www.thzhishu.com/manageredirect/';
            let orderUrl = encodeURIComponent(`${host}storeorder`);

            if (data.meta.code === 200) {
                let origin = data.data;
                let replaceUrl = origin.replace('STATE', `url-${orderUrl}@orderId-${orderid}@source-${source}`);
                window.location.replace(replaceUrl);
            } else {
                this.responseErrorHandler(data, true);
            }
        }, (err) => console.error(err));
    }

}
